import {
  Badge,
  Box,
  Button,
  HStack,
  Table,
  Tbody,
  Td,
  Th,
  Thead,
  Tooltip,
  Tr,
  VStack,
} from "@hope-ui/solid";
import { createSignal, For } from "solid-js";
import {
  useFetch,
  useListFetch,
  useManageTitle,
  useRouter,
  useT,
} from "~/hooks";
import { handleRresp, notify, r } from "~/utils";
import { PageResp, UserPermissions, User, UserMethods } from "~/types";
import { DeletePopover } from "../common/DeletePopover";

const Role = (props: { role: number }) => {
  const roles = [
    { name: "general", color: "info" },
    { name: "guest", color: "neutral" },
    { name: "admin", color: "accent" },
  ];
  return (
    <Badge colorScheme={roles[props.role].color as any}>
      {roles[props.role].name}
    </Badge>
  );
};

const Permissions = (props: { user: User }) => {
  const t = useT();
  const color = (can: boolean) => `$${can ? "success" : "danger"}9`;
  return (
    <HStack spacing="$0_5">
      <For each={UserPermissions}>
        {(item, i) => (
          <Tooltip label={t(`users.permissions.${item}`)}>
            <Box
              boxSize="$2"
              rounded="$full"
              bg={color(UserMethods.can(props.user, i()))}
            ></Box>
          </Tooltip>
        )}
      </For>
    </HStack>
  );
};

const Users = () => {
  const t = useT();
  useManageTitle("manage.sidemenu.users");
  const { to } = useRouter();
  const [getUsersLoading, getUsers] = useFetch(() => r.get("/admin/user/list"));
  const [users, setUsers] = createSignal<User[]>([]);
  const refresh = async () => {
    const resp: PageResp<User> = await getUsers();
    handleRresp(resp, (data) => setUsers(data.content));
  };
  refresh();

  const [deleting, deleteUser] = useListFetch((id: number) =>
    r.post(`/admin/user/delete?id=${id}`)
  );
  return (
    <VStack spacing="$2" alignItems="start" w="$full">
      <HStack spacing="$2">
        <Button
          colorScheme="accent"
          loading={getUsersLoading()}
          onClick={refresh}
        >
          {t("global.refresh")}
        </Button>
        <Button
          onClick={() => {
            to("/@manage/users/add");
          }}
        >
          {t("global.add")}
        </Button>
      </HStack>
      <Box w="$full" overflowX="auto">
        <Table highlightOnHover dense>
          <Thead>
            <Tr>
              <For each={["username", "base_path", "role", "permission"]}>
                {(title) => <Th>{t(`users.${title}`)}</Th>}
              </For>
              <Th>{t("global.operations")}</Th>
            </Tr>
          </Thead>
          <Tbody>
            <For each={users()}>
              {(user) => (
                <Tr>
                  <Td>{user.username}</Td>
                  <Td>{user.base_path}</Td>
                  <Td>
                    <Role role={user.role} />
                  </Td>
                  <Td>
                    <Permissions user={user} />
                  </Td>
                  <Td>
                    <HStack spacing="$2">
                      <Button
                        onClick={() => {
                          to(`/@manage/users/edit/${user.id}`);
                        }}
                      >
                        {t("global.edit")}
                      </Button>
                      <DeletePopover
                        name={user.username}
                        loading={deleting() === user.id}
                        onClick={async () => {
                          const resp = await deleteUser(user.id);
                          handleRresp(resp, () => {
                            notify.success(t("global.delete_success"));
                            refresh();
                          });
                        }}
                      />
                    </HStack>
                  </Td>
                </Tr>
              )}
            </For>
          </Tbody>
        </Table>
      </Box>
    </VStack>
  );
};

export default Users;
